/*
 * $Id$
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.

 */

#include "hbclass.ch"
#include "common.ch"
#include "hbqtgui.ch"
#include "hmg.ch"

/*----------------------------------------------------------------------*/

memvar oCurrentPage

/*----------------------------------------------------------------------*/

CLASS BASIC FROM GLOBSHARED

   // Internal Data
   DATA aBackColor                                INIT   {255,255,255}
   DATA lBold                                     INIT   NIL
   DATA lBreak                                    INIT   NIL
   DATA cClass                                    INIT   "BASIC"
   DATA nCol                                      INIT   0
   DATA aControls                                 INIT   {}
   DATA lCreated                                  INIT   .F.
   DATA oCursor                                   INIT   NIL
   DATA xCursor                                   INIT   0
   DATA cCaption                                  INIT   ''
   DATA oFont                                     INIT   NIL
   DATA aFontColor                                INIT   {0,0,0}
   DATA cFontName                                 INIT   NIL
   DATA nFontSize                                 INIT   NIL
   DATA cGripperText                              INIT   NIL
   DATA nHeight                                   INIT   NIL
   DATA cIcon                                     INIT   NIL
   DATA lItalic                                   INIT   NIL
   DATA cName                                     INIT   NIL
   DATA bOnClick                                  INIT   NIL
   DATA bOnGotFocus                               INIT   NIL
   DATA bOnLostFocus                              INIT   NIL
   DATA bOnMouseDrag                              INIT   NIL
   DATA bOnMouseMove                              INIT   NIL
   DATA bOnMouseClick                             INIT   NIL
   DATA cParent                                   INIT   NIL
   DATA oParent                                   INIT   NIL
   DATA oQTObject                                 INIT   NIL
   DATA nRow                                      INIT   0
   DATA lSizable                                  INIT   .T.
   DATA lStrikeout                                INIT   NIL
   DATA cStyleSheet                               INIT   NIL
   DATA lUnderline                                INIT   NIL
   DATA lVisible                                  INIT   .T.
   DATA nWidth                                    INIT   NIL

   // Properties
   METHOD BackColor                               SETGET
   METHOD Break                                   SETGET
   METHOD Col                                     SETGET
   METHOD Cursor                                  SETGET
   METHOD CursorPos                               SETGET
   METHOD Focused( lValue )                       INLINE ::FocusedControl( lValue )
   METHOD FocusedControl                          SETGET
   METHOD Font                                    SETGET
   METHOD Height                                  SETGET
   METHOD FontBold                                SETGET
   METHOD FontColor                               SETGET
   METHOD FontItalic                              SETGET
   METHOD FontName                                SETGET
   METHOD FontSize                                SETGET
   METHOD FontStrikeout                           SETGET
   METHOD FontUnderLine                           SETGET
   METHOD GripperText                             SETGET
   METHOD Icon                                    SETGET
   METHOD MaxHeight                               SETGET
   METHOD MaxWidth                                SETGET
   METHOD MinHeight                               SETGET
   METHOD MinWidth                                SETGET
   METHOD Name                                    SETGET
   METHOD Row                                     SETGET
   METHOD StyleSheet                              SETGET
   METHOD Title                                   SETGET
   METHOD Width                                   SETGET
   METHOD Sizable                                 SETGET
   METHOD Visible                                 SETGET

   // Events
   METHOD MouseClickProcess                       SETGET
   METHOD OnClick                                 SETGET
   METHOD OnMouseClick                            SETGET
   METHOD OnMouseDrag                             SETGET
   METHOD OnMouseMove                             SETGET

   // Methods
   METHOD SendEvent
   METHOD Show
   METHOD Hide
   METHOD Release
   METHOD AddData
   METHOD DelData
   METHOD CreatePendingChildControls

   METHOD New
   METHOD Create

   DATA  aChilds        INIT hb_hash()

   ERROR HANDLER XerrorHandler( uParam )

// method: please preserve alphabetic order. New and Create must be the first
   METHOD StartHash
   METHOD AddItem
   METHOD GetItem
   METHOD IsDefined
   METHOD ItemCount                       INLINE LEN( ::aChilds )
   METHOD RemoveItem

ENDCLASS

/*----------------------------------------------------------------------*/
//                             Properties
/*----------------------------------------------------------------------*/

METHOD Name( cValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::cName
   ELSEIF Pcount() == 1
      IF ::cName != cValue

         // Remove the old name from the parent object stack
         IF ! Hb_IsNil( ::cName )
            ::oParent:DelData( ::cName )
         ENDIF

         ::cName := cValue
         ::oParent:AddData( ::cName, Self )

      ENDIF
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Row( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:y()
   ELSEIF Pcount() == 1
      ::oQTObject:move( ::oQTObject:x(), nValue )
      ::nRow := nValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Col( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:x()
   ELSEIF Pcount() == 1
      ::oQTObject:move( nValue, ::oQTObject:y() )
      ::nCol := nValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Width( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:width()
   ELSEIF Pcount() == 1
      ::oQTObject:resize( nValue, ::oQTObject:height() )
      ::nWidth := nValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Height( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:height()
   ELSEIF Pcount() == 1
      ::oQTObject:resize( ::oQTObject:width(), nValue )
      ::nHeight := nValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD MaxWidth( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:maximumWidth()
   ELSEIF Pcount() == 1
      ::oQTObject:setMaximumWidth( nValue )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD MinWidth( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:minimumWidth()
   ELSEIF Pcount() == 1
      ::oQTObject:setMinimumWidth( nValue )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD MaxHeight( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:maximumHeight()
   ELSEIF Pcount() == 1
      ::oQTObject:setMaximumHeight( nValue )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD MinHeight( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:minimumHeight()
   ELSEIF Pcount() == 1
      ::oQTObject:setMinimumHeight( nValue )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Font( xValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont
   ELSEIF Pcount() == 1
      IF Valtype( xValue ) == "C"
         RETURN ::fontName( xValue )
      ELSEIF Valtype( xValue ) == "O"
         ::oFont := xValue
         ::oQTObject:setFont( ::oFont )
      ENDIF
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontName( cValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont:Family()
   ELSEIF Pcount() == 1
      ::oFont:setFamily( cValue )
      ::oQTObject:setFont( ::oFont )
      ::cFontName := cValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontSize( nValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont:pointSize()
   ELSEIF Pcount() == 1
      ::oFont:setPointSize( nValue )
      ::oQTObject:setFont( ::oFont )
      ::nFontSize := nValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontBold( lValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont:bold()
   ELSEIF Pcount() == 1
      ::oFont:setBold( lValue )
      ::oQTObject:setFont( ::oFont )
      ::lBold := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontItalic( lValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont:italic()
   ELSEIF Pcount() == 1
      ::oFont:setItalic( lValue )
      ::oQTObject:setFont( ::oFont )
      ::lItalic := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontUnderline( lValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oFont:underLine()
   ELSEIF Pcount() == 1
      ::oFont:setUnderLine( lValue )
      ::oQTObject:setFont( ::oFont )
      ::lUnderline := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontStrikeout( lValue ) CLASS BASIC

   IF Pcount() == 0
      ::oFont:StrikeOut()
   ELSEIF Pcount() == 1
      ::oFont:setStrikeOut( lValue )
      ::oQTObject:setFont( ::oFont )
      ::lStrikeout := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FontColor( aValue ) CLASS BASIC

   LOCAL cStyle

   IF Pcount() == 0
      RETURN ::aFontColor
   ELSEIF VALTYPE( aValue ) == 'A'
      ::aFontColor := aValue
      cStyle := "color:#" + hb_numtohex( ::aFontColor[1], 2 ) + hb_numtohex( ::aFontColor[2], 2 ) + hb_numtohex( ::aFontColor[3], 2 ) + ";"
      Self:StyleSheet( cStyle )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD BackColor( aValue ) CLASS BASIC

   LOCAL cStyle

   IF PCOUNT() == 0
      RETURN ::aBackColor
   ELSEIF VALTYPE( aValue ) == 'A'
      ::aBackColor := aValue
      cStyle := "background-color:#" + hb_numtohex( ::aBackColor[1], 2 ) + hb_numtohex( ::aBackColor[2], 2 ) + hb_numtohex( ::aBackColor[3], 2 ) + ";"
      Self:StyleSheet( cStyle )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Sizable( lValue ) CLASS BASIC

   LOCAL oLayout

   oLayout := ::oQTObject:layout()
   IF PCOUNT() == 0
      RETURN ::lSizable
   ELSEIF VALTYPE( lValue ) == "L"
      ::lSizable := lValue
      IF ::lSizable == .T.
         oLayout:setSizeConstraint( QLayout_SetMinAndMaxSize )
         ::oQTObject:setLayout( oLayout )
      ELSE
         ::oQTObject:setFixedSize( ::oQTObject:size() )
      ENDIF
      ::oQTObject:show()
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Visible( lValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::oQTObject:isVisible()
   ELSEIF Pcount() == 1
      ::oQTObject:setVisible( lValue )
      ::lVisible := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD GripperText( cValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::cGripperText
   ELSEIF Pcount() == 1
      ::cGripperText := cValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Icon( cValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::cIcon
   ELSEIF VALTYPE( cValue ) == "C"
      ::cIcon := cValue
      ::oQTObject:setWindowIcon( ::cIcon )
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Break( lValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::lBreak
   ELSEIF Pcount() == 1
      ::lBreak := lValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD StyleSheet( cValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::cStyleSheet
   ELSEIF VALTYPE( cValue ) == "C"
      ::cStyleSheet := StyleUpdater( ::cStyleSheet, cValue )
      ::oQtObject:setStyleSheet( ::cStyleSheet )
      ::oQtObject:repaint()
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Title( cValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::oQTObject:windowTitle()
   ELSEIF VALTYPE( cValue ) == "C"
      ::cCaption := cValue
      ::oQTObject:setWindowTitle( ::cCaption )
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
//                                Methods
/*----------------------------------------------------------------------*/

METHOD SendEvent( nKey, nModifier, oQtActWin ) CLASS BASIC

  LOCAL oKey

  nKey      := IIF( VALTYPE( nKey )      == "N", nKey,      Qt_Key_Tab )            // by default use Tab key
  nModifier := IIF( VALTYPE( nModifier ) == "N", nModifier, Qt_NoModifier )         // by default don't use modifier (Shit, Ctrl, Alt )
  oQtActWin := IIF( VALTYPE( oQtActWin ) == "U", HMGAPP():CurrQtWin(), oQtActWin )  // by default use Qt Active Window

// see Qt QKeyEvent at http://doc.qt.nokia.com/latest/qkeyevent.html
  oKey      := QKeyEvent( QEvent_KeyPress, nKey, nModifier, 0, 0)

// see Qt sendEvent at http://doc.qt.nokia.com/latest/eventsandfilters.html
  ::s_oQtApp:sendEvent( oQtActWin, oKey )
// see Qt processEvents at http://doc.qt.nokia.com/latest/qcoreapplication.html
  ::s_oQtApp:processEvents()

RETURN .T.

/*----------------------------------------------------------------------*/

METHOD Show() CLASS BASIC

   ::oQTObject:show()

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Hide() CLASS BASIC

   ::oQTObject:hide()

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Release() CLASS BASIC

   // Some QT classes do not have close()
   IF ! ( ::cClass == "MENUITEM"   .OR. ;
          ::cClass == "SOUND"      .OR. ;
          ::cClass == "TIMER"      .OR. ;
          ::cClass == "LAYOUTBOX"  .OR. ;
          ::cClass == "LAYOUTFORM" .OR. ;
          ::cClass == "LAYOUTGRID" )

      // release QT object
      IF ! hb_IsNil( ::oQTObject )
         ::oQTObject:close()
      ENDIF

      ::oParent:DelData( ::cName )
      ::oQTObject := NIL

   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD AddData( cName , oObject ) CLASS BASIC

   // __objAddData( Self , LOWER( cName ) )
   ::AddItem( cName, oObject )

   // ::&cName := oObject

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD DelData( cName ) CLASS BASIC

   IF ::IsDefined( cName )
       ::Removeitem( cName )
   ENDIF
   //IF __objHasMsg( Self, LOWER( cName ) )      // This tells that cName exists in the object.
   //   ::&cName := NIL
   //ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
#include "hbtrace.ch"
METHOD CreatePendingChildControls() CLASS BASIC
   LOCAL i

HB_TRACE( HB_TR_ALWAYS, "entro  Create Child Controls" )
   FOR i := 1 TO Len( ::aControls )
      IF !::aControls[i]:lCreated
         ::aControls[i]:Create()
      ENDIF
   NEXT i

HB_TRACE( HB_TR_ALWAYS, "esco  Create Child Controls" )
   RETURN NIL

/*----------------------------------------------------------------------*/
//                             Events
/*----------------------------------------------------------------------*/

METHOD MouseClickProcess( e ) CLASS BASIC
// LOCAL oMouseEvent := HB_QMouseEvent():from( e )
   LOCAL oMouseEvent := e

   IF oMouseEvent:button() == Qt_LeftButton
      EVAL( ::bOnMouseClick )
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnClick( bValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::bOnClick
   ELSEIF Pcount() == 1
      ::bOnClick := bValue
      ::oQTObject:disConnect( "clicked()" )
      ::oQTObject:connect( "clicked()" , ::bOnClick )
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD OnMouseClick( bValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::bOnMouseClick
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnMouseClick := bValue
      IF .NOT. ::oQTObject:connect( QEvent_MouseButtonRelease , { |e| ::MouseClickProcess( e ) } )
         // Error
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnMouseDrag( bValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::bOnMouseDrag
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnMouseDrag := bValue
      IF .NOT. ::oQTObject:connect( QEvent_DragEnter , { || EVAL( ::bOnMouseDrag ) } )
         // Error
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnMouseMove( bValue ) CLASS BASIC

   IF PCOUNT() == 0
      RETURN ::bOnMouseMove
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnMouseMove := bValue
      ::oQTObject:setMouseTracking( .T. )
      IF .NOT. ::oQTObject:connect( QEvent_MouseMove , { || EVAL( ::bOnMouseMove ) } )
         // Error
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
//                        Generic Create Method
/*----------------------------------------------------------------------*/

METHOD Create() CLASS BASIC

   ::lCreated := .T.

   // Create inner controls
   ::CreatePendingChildControls()

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD New( cName, oParent ) CLASS BASIC

   // check if HMGAPP is started
   IF ::s_lQtStarted == .F.
      HMGAPP():Start()
   ENDIF

   ::StartHash() // called here since it's new method is called too late

   // If the object is a cell in a grid, returns an empty object, with no parent.

   IF ::s_lGridCellFlag == .F.

      IF PCOUNT() == 0 .OR. VALTYPE( cName ) <> "C"
         ::cName := '_HMG_OBJECT_' + hb_ntos( ::s_nObjectCounter++ )
      ELSE
         ::cName  := cName
      ENDIF

      // In a nested WITH OBJECT structure the parent ( container ) of a control is the
      // previous object in the stack.
      // In HMG4 syntax, the NEW() method is always called in the same line of the WITH OBJECT statement
      // and the current object in the nested structure is the parent.
      IF hb_isnumeric( oParent ) .AND. oParent = 0    // If numeric 0 ( zero ) force no parent.
         ::oParent    := Basic():New()
      ELSEIF hb_isobject( oParent )                   // If parent object is passed explicitly, use it.
         ::oParent    := oParent
      ELSEIF hb_isobject( oParent := HB_QWith() )
         ::oParent    := oParent
      ELSE
         ::oParent := ::s_oDefaultParent
      ENDIF

      IF hb_isobject( ::oParent ) == .T.
         AADD( ::oParent:aControls , Self )
         ::oParent:AddData( ::cName , Self )
      ENDIF

   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/
//                        Cursor Object Method
/*----------------------------------------------------------------------*/

METHOD Cursor( xValue ) CLASS BASIC

   IF Pcount() == 0
      RETURN ::xCursor
   ELSEIF Pcount() == 1
      IF hb_IsChar( xValue )
         /// Only bitmaps with masks are allowed.
         /// The number of parameters should be changed to 2 with bitmap and its mask...
         ::oCursor := QCursor( QBitmap( xValue ), QBitmap( xValue ), -1, -1 )
         ::oQTObject:setCursor( ::oCursor )

      ELSEIF hb_IsNumeric( xValue )
         ::oCursor := QCursor( xValue )
         ::oQTObject:setCursor( ::oCursor )

      ENDIF

      ::xCursor := xValue
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD CursorPos( nRow, nCol ) CLASS BASIC

   LOCAL oPos

   IF hb_IsObject( ::oCursor )

      IF Pcount() == 0
         oPos := ::oCursor:pos()
         RETURN { ::oQTObject:mapFromGlobal( oPos ):y, ::oQTObject:mapFromGlobal( oPos ):x }

      ELSEIF Pcount() == 2
         oPos := QPoint( nCol, nRow )
         ::oCursor:setPos( ::oQTObject:mapToGlobal( oPos ):x, ::oQTObject:mapToGlobal( oPos ):y )

      ENDIF
   ENDIF

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD FocusedControl( cValue ) CLASS BASIC

   LOCAL oQTObject, nI

   IF PCOUNT() == 0
      FOR nI := 1 TO LEN( ::aControls )
         oQTObject := ::aControls[nI]:oQTObject
         IF oQTObject:hasFocus()
            RETURN ::aControls[nI]:cName
         ENDIF
      NEXT nI
   ELSEIF VALTYPE( cValue ) == "C"
      FOR nI := 1 TO LEN( ::aControls )
         oQTObject := ::aControls[nI]:oQTObject
         IF ::aControls[nI]:cName == cValue
            oQTObject:setFocus()
            EXIT
         ENDIF
      NEXT nI
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/



#include "Error.ch"

METHOD StartHash( ) CLASS BASIC

? "myclass:new()"
   ::aChilds := hb_Hash()
   hb_HCaseMatch( ::aChilds, .F. )     // No case sensitive
   hb_HKeepOrder( ::aChilds, .T. )     // inserting order

RETURN Self

/*..............................................................................
   AddItem
..............................................................................*/
METHOD AddItem( cName, oValue ) CLASS BASIC

   // oValue defaults to NIL if not specified
   hb_HSet( ::aChilds, cName, oValue )

RETURN Self

/*..............................................................................
   GetItem
..............................................................................*/
METHOD GetItem( cNameToFind ) CLASS BASIC
// will raise an error if cNameToFind is not present
// we should add hb_HHaskey check if we don't want RTE
RETURN ( hb_HGet( ::aChilds, cNameToFind ) )

/*..............................................................................
   IsDefined
..............................................................................*/
METHOD IsDefined( cNameToFind ) CLASS BASIC

RETURN hb_HHaskey( ::aChilds, cNameToFind )

/*..............................................................................
   RemoveItem
..............................................................................*/
METHOD RemoveItem( cName ) CLASS BASIC

   // unset the item, to force item destructor
   hb_HSet( ::aChilds, cName, NIL )
   hb_HDel( ::aChilds, cName )
   
RETURN Self

/*..............................................................................
   XerrorHandler
..............................................................................*/
METHOD XerrorHandler( uParam ) CLASS BASIC

   LOCAL cMsg := __GetMessage()
   LOCAL uRet, oErr

   IF SUBSTR( cMsg, 1, 1 ) == "_"
      cMsg := SUBSTR( cMsg, 2 )
   ENDIF
   
   IF hb_HHaskey( ::aChilds, cMsg ) 
      IF uParam == NIL
          uRet  := hb_HGet( ::aChilds, cMsg )
      ELSE
          hb_HSet( ::aChilds, cMsg, uParam )
      ENDIF
   ELSE
      oErr := ERRORNEW()
      oErr:Args          := { Self, cMsg, uParam }
      oErr:CanDefault    := .F.
      oErr:CanRetry      := .F.
      oErr:CanSubstitute := .T.
      oErr:Description   := "Invalid class member"
      oErr:GenCode       := EG_NOVARMETHOD
      oErr:Operation     := "BASIC:" + cMsg
      oErr:Severity      := ES_ERROR
      oErr:SubCode       := -1
      oErr:SubSystem     := "BASIC"
      uRet := EVAL( ERRORBLOCK(), oErr )
   ENDIF

RETURN uRet


